@@ -31,21 +31,22 @@ makeLowerUpper <- function(canon.init, ## cf calls: ~ in user scale, must be a f
                            rhorange=NULL,
                            RHOMAX,NUMAX) {
   lower <- upper <- init.optim   ## init.optim not further used...
-  if (corr.model %in% c("SAR_WWt","adjacency","ar1")) { ## adjacency model
+  if (corr.model %in% c("SAR_WWt","adjacency")) { ## adjacency model
+    eps <- (rhorange[2L]-rhorange[1L])/(2e6)  
     lower$rho <- user.lower$rho ## no transfo for adjacency model
-    if (is.null(lower$rho)) lower$rho <- rhorange[1L] ## may remain NULL  
+    if (is.null(lower$rho)) lower$rho <- rhorange[1L]+eps ## may remain NULL  
     upper$rho <- user.upper$rho ## no transfo again
-    if (is.null(upper$rho)) upper$rho <- rhorange[2L]
+    if (is.null(upper$rho)) upper$rho <- rhorange[2L]-eps
   } else {
-    if (corr.model=="AR1") {
+    if (corr.model %in% c("AR1","ar1")) {
       if ( ! is.null(canon.init$ARphi)) {
         ARphi <- user.lower$ARphi
-        if (is.null(ARphi)) ARphi <- -0.999999
+        if (is.null(ARphi)) ARphi <- -1 + 1e-6
         lower$ARphi <- ARphi
         ARphi <- user.upper$ARphi
-        if (is.null(ARphi)) ARphi <- 0.999999
+        if (is.null(ARphi)) ARphi <- 1 - 1e-6
         upper$ARphi <- ARphi
-      }    
+      }
     } else { ## then Matern model....
       if (! is.null(canon.init$rho)) {
         rho <- user.lower$rho
@@ -96,34 +97,49 @@ makeLowerUpper <- function(canon.init, ## cf calls: ~ in user scale, must be a f
   }
   if (! is.null(canon.init$phi)) {
     phi <- user.lower$phi
-    if (is.null(phi)) phi <- canon.init$phi/1000
+    if (is.null(phi)) phi <- max(1e-6,canon.init$phi/1e5)
     lower$trPhi <- dispFn(phi)
     phi <- user.upper$phi
-    if (is.null(phi)) phi <- canon.init$phi*1000
+    if (is.null(phi)) phi <-  min(1e8,canon.init$phi*1e7)
     ## if phi is badly initialized then it gets a default which may cause hard to catch problems in the bootstrap...
     upper$trPhi <- dispFn(phi)
   }
   if (! is.null(canon.init$lambda)) {
     lambda <- user.lower$lambda
-    if (is.null(lambda)) lambda <- canon.init$lambda/1000
+    if (is.null(lambda)) lambda <- pmax(1e-6,canon.init$lambda/1e5)
     lower$trLambda <- dispFn(lambda)
     lambda <- user.upper$lambda
-    if (is.null(lambda)) lambda <- canon.init$lambda*1000
+    if (is.null(lambda)) lambda <- pmin(1e8,canon.init$lambda*1e7)
     upper$trLambda <- dispFn(lambda)
   }
-  return(list(lower=lower,upper=upper))
+  if (! is.null(canon.init$COMP_nu)) {
+    COMP_nu <- user.lower$COMP_nu
+    if (is.null(COMP_nu)) COMP_nu <- min(canon.init$COMP_nu/2,0.05)
+    lower$COMP_nu <- COMP_nu
+    COMP_nu <- user.upper$COMP_nu
+    if (is.null(COMP_nu)) COMP_nu <- max(canon.init$COMP_nu*10,10)
+    upper$COMP_nu <- COMP_nu
+  } else if (! is.null(canon.init$NB_shape)) {
+    NB_shape <- user.lower$NB_shape
+    if (is.null(NB_shape)) NB_shape <- 1e-6
+    lower$NB_shape <- NB_shape
+    NB_shape <- user.upper$NB_shape
+    if (is.null(NB_shape)) NB_shape <- max(100*canon.init$NB_shape,1e6)
+    upper$NB_shape <- NB_shape
+  }
+  ## names() to make sure the order of elements match; remove extra stuf (=> either ARphi or rho fr ar1)
+  return(list(lower=lower[names(init.optim)],upper=upper[names(init.optim)])) 
 }
 
 checkDistMatrix <- function(distMatrix,data,coordinates) {
-  classDistm <- class(distMatrix)
-  if ( ! classDistm %in% c("matrix","dist")) {
-    message(paste("(!) 'distMatrix' argument appears to be a '",classDistm,"',",sep=""))
-    stop("not a 'matrix' or 'dist'. Check the input. I exit.")
-  }
+  if (inherits(distMatrix,"dist")) {
+    usernames <- labels(distMatrix)
+  } else if (inherits(distMatrix,"matrix")) {
+    usernames <- rownames(distMatrix)
+  } else message(paste("(!) 'distMatrix' is neither a 'matrix' or 'dist' object. Check the input. I exit."))
   ## chol() fails on distances matrices with repeated locations (which are pos SD)... but chol() not used by default
   ## the following code assumes that distMatrix deals only with unique locations, and checks this
   ## HENCE ******* distMatrix must refer to unique values of a grouping variable *********
-  usernames <- rownames(distMatrix)
   checknames <- all(sapply(usernames,function(v) {v %in% rownames(data)})) ## 
   if (!checknames) {
     warning("The rownames of 'distMatrix' are not rownames of the 'data'. Further checking of 'distMatrix' is not possible.")
@@ -326,11 +342,13 @@ SEMdiagnosticPlot <- function(Krigobj,MAX,titlemain, optr) {
   invisible(NULL)
 }
 
+## FR->FR j'ai oublié setControl()...
 reformat_verbose <- function(verbose,For) {
   if (is.null(verbose)) verbose <- logical(0)
   if (is.list(verbose)) stop("is.list(verbose)")
   if (is.na(verbose["trace"])) verbose["trace"] <- FALSE
   if (is.na(verbose["SEM"])) verbose["SEM"] <- FALSE
+  if (For=="corrHLfit" && is.na(verbose["objective"])) verbose["objective"] <- FALSE
   if (is.na(verbose["warn"])) verbose["warn"] <- switch(For, "HLCor" = TRUE, "corrHLfit" = FALSE)
   summstring <- paste(For,"Summary",sep="")
   verbose[summstring] <- verbose["summary"]
@@ -369,22 +387,27 @@ calc_maxrange <- function(rho.size,distMatrix=NULL,uniqueGeo=NULL,rho_mapping,di
   return(maxrange)
 }
 
-calc_init_dispPars <- function(init,init.optim,init.HLfit,ranFix) {
+calc_inits_dispPars <- function(init,init.optim,init.HLfit,ranFix) {
   ## does not modify init.HLfit, but keeps its original value. Also useful to keep ranFix for simple and safe coding
-  if (is.null(getPar(ranFix,"lambda"))) { ## no ranFix$lambda: process init.optim
-    init$lambda <- init.optim$lambda 
-    if (!is.null(init$lambda)) {
-      if (init$lambda<1e-4) init$lambda <- 1e-4
-      init.optim$trLambda <- dispFn(init$lambda) 
-      init.optim$lambda <- NULL
-    }
-  } else { ## ranFix$lambda present, do NOT put it in init.optim
-    if (!is.null(init.optim$lambda)) stop("(!) Arguments 'ranFix$lambda' and 'init$lambda' conflict with each other.")  
-  } 
+  init$lambda <- init.optim$lambda 
+  fixedlambda <- getPar(ranFix,"lambda") ## FR->FR his comes from user arg, not from preprocessing
+  if (!is.null(fixedlambda)) {
+    whichNA <- which(is.na(fixedlambda))
+    if (length(init$lambda)==length(whichNA)) {
+      if (length(whichNA)>0L) names(init$lambda) <- whichNA
+    } else if (length(init$lambda)<length(whichNA)) {
+      ## init$lambda should already have names...
+    } else stop("'fixed lambda' and 'init lambda' arguments conflict with each other.")
+  }
+  if (!is.null(init$lambda)) {
+    init$lambda[init$lambda<1e-4] <- 1e-4
+    init.optim$trLambda <- dispFn(init$lambda) 
+    init.optim$lambda <- NULL
+  }
   if (is.null(getPar(ranFix,"phi"))) {
     init$phi <- init.optim$phi 
     if (!is.null(init$phi)) {
-      if (init$phi<1e-4) init$phi <- 1e-4
+      init$phi[init$phi<1e-4] <- 1e-4
       init.optim$trPhi <- dispFn(init$phi)
       init.optim$phi <- NULL
     }
@@ -438,13 +461,27 @@ calc_inits_nu <- function(init,init.optim,init.HLfit,ranFix,control.dist,optim.s
   return(list(init=init,init.optim=init.optim,init.HLfit=init.HLfit,ranFix=ranFix))
 }
 
-calc_inits_Auto_rho <- function(init,init.optim,init.HLfit,ranFix,rhorange) {
-  if (is.null(getPar(ranFix,"rho")) && (! is.numeric(init.HLfit$rho))) { ## $rho NULL or NA
-    if (is.null(init.optim$rho)) init.optim$rho <- mean(rhorange)
-  }
-  if ( (! is.null(init.HLfit$rho)) && (! is.numeric(init.HLfit$rho))) { ## init.HLfit$rho is NA
-    init.HLfit$rho <- init.optim$rho
-    init.optim$rho <- NULL 
+calc_inits_Auto_rho <- function(init,init.optim,init.HLfit,ranFix,rhorange,For) {
+  if (is.null(getPar(ranFix,"rho"))) { ## $rho NULL or NA
+    if (For=="corrHLfit") { ## with corrHLfit, defaul is outer optim. We provide init.optim
+      if (is.null(init.HLfit$rho)) { ## default init.HLfit
+        if (! is.numeric(init.optim$rho)) init.optim$rho <- mean(rhorange)
+      } else if ( ! is.numeric(init.HLfit$rho) ) { ## non-default: inner optim, but no numeric init  
+        init.HLfit$rho <- mean(rhorange)
+        init.optim$rho <- NULL 
+      } else { ## non-default: inner optim, already with init value
+        init.optim$rho <- NULL 
+      }
+    } else if (For=="fitme") { ## with fitme, default is inner optim. We provide init.HLfit
+      if (is.null(init.optim$rho)) { ## default init.optim
+        if (! is.numeric(init.HLfit$rho)) init.HLfit$rho <- mean(rhorange)
+      } else if ( ! is.numeric(init.optim$rho) ) { ## non-default: outer optim, but no numeric init  
+        init.optim$rho <- mean(rhorange) 
+        init.HLfit$rho <- NULL
+      } else { ## non-default: inner optim, already with init value
+        init.HLfit$rho <- NULL 
+      }
+    }
   }
   return(list(init=init,init.optim=init.optim,init.HLfit=init.HLfit,ranFix=ranFix))
 }
@@ -473,7 +510,7 @@ calc_inits_geostat_rho <- function(init,init.optim,init.HLfit,ranFix,maxrange,op
 }
 
 calc_inits <- function(init.optim,init.HLfit,ranFix,corr.model,rhorange=NULL,maxrange=NULL,
-                       optim.scale,control.dist,RHOMAX,NUMAX) { 
+                       optim.scale,control.dist,RHOMAX,NUMAX,For) { 
   inits <- as.list(match.call()[c("init.optim","init.HLfit","ranFix")])
   inits <- c(inits,list(init=list()))
   if (corr.model %in% c("Matern")) {
@@ -483,14 +520,17 @@ calc_inits <- function(init.optim,init.HLfit,ranFix,corr.model,rhorange=NULL,max
     inits <- do.call(calc_inits_nu,arglist)
     # Nugget: remains NULL through all computations if init.optim$Nugget is NULL
     if (is.null(getPar(ranFix,"Nugget"))) { inits$init["Nugget"] <- init.optim$Nugget }  
-  } else if ( corr.model  %in% c("SAR_WWt","adjacency","ar1") ) { 
-    arglist <- c(inits,list(rhorange=rhorange))
+  } else if ( corr.model  %in% c("SAR_WWt","adjacency") ) { 
+    arglist <- c(inits,list(rhorange=rhorange,For=For))
     inits <- do.call(calc_inits_Auto_rho,arglist)
-  } else if (corr.model=="AR1") {
+  } else if (corr.model %in% c("AR1","ar1")) {
     inits <- do.call(calc_inits_ARphi,inits)
   }  
   # phi, lambda
-  inits <- do.call(calc_init_dispPars,inits)
+  inits <- do.call(calc_inits_dispPars,inits)
+  # GLM family parameters
+  inits$init$COMP_nu <- inits$init.optim$COMP_nu ## may be NULL. No checks needed
+  inits$init$NB_shape <- inits$init.optim$NB_shape ## may be NULL. No checks needed
   return(inits)
 }
 
